/* $LAAS: client.c 2008/07/21 15:55:17 mallet $ */

/* 
 * Copyright (c) 2008 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *					Anthony Mallet on Fri Jul  4 2008
 */

#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <sys/types.h>

#include <pthread.h>

#include "evart-client.h"


/* --- local data ---------------------------------------------------- */

static void *	reader(void *data);


/* --- main ---------------------------------------------------------- */

int
main(int argc, char * const argv[])
{
  pthread_t rth;
  char line[1024];
  int i, s;

  /* options */
  static const char *usage =
    "usage: %s [--host evas_host] [--port port_number]\n";
  static const struct option long_options[] = {
    { "host", 1, NULL, 'h' },
    { "port", 1, NULL, 'p' },
    { NULL, 0, NULL, 0 }
  };

  while(1) {
    s = getopt_long(argc, argv, "h:p:", long_options, NULL);
    if (s == -1) break;

    switch (s) {
      case 'h':	evas_sethost(optarg); break;
      case 'p': evas_setport(atoi(optarg)); break;

      case '?':
      default:
	printf(usage, argv[0]);
	exit(2);
    }
  }
  if (optind < argc) {
    printf(usage, argv[0]);
    exit(2);
  }

  /* initialize internal data by sending a hello msg */
  evas_text("hello");

  /* spawn async reader */
  s = pthread_create(&rth, NULL, reader, NULL);
  if (s) exit(2);

  /* read commands from stdin */
  printf(PACKAGE_STRING " ready\n");
  while(fgets(line, sizeof(line), stdin)) {
    if (strrchr(line, '\n')) *strrchr(line, '\n') = '\0';

#define HELLO			"hello"
#define BODY_LIST		"list body"
#define BODY_MARKERS_LIST	"list markers "
#define BODY_MARKERS		"markers "
#define UNREC_MARKERS		"unrec markers "

    if (!strncmp(line, BODY_LIST, strlen(BODY_LIST))) {
      const evas_body_list_t *bl = evas_body_list();
      if (!bl) { printf("* error\n"); continue; }

      for(i=0; i<bl->nbodies; i++)
	printf("* %d:%s\n", i, bl->bodies[i]);
      printf("* end of list\n");

    } else if (!strncmp(line, BODY_MARKERS_LIST, strlen(BODY_MARKERS_LIST))) {
      const evas_body_markers_list_t *ml =
	evas_body_markers_list(atoi(&line[strlen(BODY_MARKERS_LIST)]));
      if (!ml) { printf("* error\n"); continue; }

      printf("* body %d:%s\n", ml->index, ml->name);
      for(i=0; i<ml->nmarkers; i++)
	printf("* %d:%s\n", i, ml->markers[i]);
      printf("* end of list\n");

    } else if (!strncmp(line, BODY_MARKERS, strlen(BODY_MARKERS))) {
      if (line[strlen(BODY_MARKERS)] == '-')
	evas_body_markers(atoi(&line[strlen(BODY_MARKERS)+1]), EVAS_OFF);
      else
	evas_body_markers(atoi(&line[strlen(BODY_MARKERS)]), EVAS_ON);

    } else if (!strncmp(line, UNREC_MARKERS, strlen(UNREC_MARKERS))) {
      if (line[strlen(UNREC_MARKERS)] == '-')
	evas_unrecognized_markers(EVAS_OFF);
      else
	evas_unrecognized_markers(EVAS_ON);

    } else if (!strncmp(line, "help", strlen("help"))) {
      printf("* Commands:\n");
      printf("%s\n", HELLO);
      printf("%s\n", BODY_LIST);
      printf("%s<body number>\n", BODY_MARKERS_LIST);
      printf("%s[-]<body number>\n", BODY_MARKERS);
      printf("%s[+-]\n", UNREC_MARKERS);
    } else
      evas_text(line);
  }
}


/* --- reader thread ------------------------------------------------- */

static void *
reader(void *data)
{
  const evas_msg_t *msg;

  while(1) {
    msg = evas_recv();

    if (msg) switch(msg->type) {
      case EVAS_NONE: break;
      case EVAS_MSG:  printf("* %s\n", msg->cmd); break;

      case EVAS_BODY_MARKERS:
	printf("* frame %d, delay %g, body %s, %d marker data\n",
	       msg->body_markers.iFrame, msg->body_markers.fDelay,
	       msg->body_markers.name, msg->body_markers.nmarkers);
	break;

      case EVAS_UNRECOGNIZED_MARKERS:
	printf("* frame %d, delay %g, %d unrecognized markers\n",
	       msg->unrecognized_markers.iFrame,
	       msg->unrecognized_markers.fDelay,
	       msg->unrecognized_markers.nmarkers);
	break;
    }
  }
}
